'''
 Filter to implement the transformation of the INPUT IMAGE in a 3D-like wireframe
 ................................................................................
 
 Release 1.0   initial
 Release 1.1   fixed restore of BG/BG colours
               added an adjustment on LCS at the very beginning to start with a well
               contrasted, luminous and saturated image
               increased the parameter values on G'MIC Bilateral to get a better convolution
               added a new parameter for the downsizing on the original image
               added a new parameter for the use to restore the colours
               added actions on the final grayscale outcome to improve contrast
               restored the image to its original size
 
 
 Flow implemented to get the final result:     
 
 1. Adjust LCS on the input drawable
 2. Duplicate the layer and set Layer Mode to GRAIN_EXTRACT
 3. Apply G'MIC Bilateral filtering
 4. Downsize the image using the user input parameter (divisor)
 5. Add a layer filled with White at 50% Opacity
 6. Apply G'MIC 3D Elevation (width and height set to the double of original image)
 7. Add a BG Layer filled with Black
 8. New layer from visible (wireframed outcome)
 9. Autostretch HSV
 10.Add contrast as per user input parameter
 11.Apply Levels auto-stretching
 12.Desaturate (luminosity)
 13.Duplicate the layer, Equalize and set opacity to 35%, Mode to DODGE
 14.Scale the image back to its original sioze
 15.If user selected to restore colors:
    15a.add a new layer with a copy of the original drawable. set mode to COLOR, opacity to 100%
 16.Flatten the image only on request of the user (default=False)   
 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
 
'''
#!/usr/bin/env python
from gimpfu import *
def WireframeMap_Lyle(inImage, inDrawable, inDivisor, inContrast, doColour, doFlatten) :

    width = inImage.width
    height = inImage.height
    # Save Context and Set up an undo group
    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    pdb.gimp_image_undo_group_start(inImage)
    # Add an Alpha Channel if missing
    alpha=pdb.gimp_drawable_has_alpha(inDrawable) 
    if alpha ==0:
        pdb.gimp_layer_add_alpha(inDrawable)
    type = inImage.base_type
    # rough adjustment on saturation/value/contrast
    #pdb.script_fu_auto_LCS_enhance(inImage, inDrawable, 0, 0, 128, 64, 152)
    pdb.python_fu_mod_LCS(inImage, inDrawable, 0, 3, 128, 64, 152)
    #  conserve the original layer as a buffer
    origBufferName = pdb.gimp_edit_named_copy(inDrawable, "myBufferName1")
    # Duplicate the Layer
    BilatLayer = inDrawable.copy()
    BilatLayer.mode = GRAIN_EXTRACT_MODE
    BilatLayer.name = "Bilateral Layer"
    BilatLayer.opacity = 100.0 
    inImage.add_layer(BilatLayer, 0)
    BilatLayer = inImage.active_layer
    # -gimp_bilateral 10,12,2,0,1,24,0
    pdb.plug_in_gmic(inImage, BilatLayer, 1,    # 1=input only active layer
    #   "-v - -gimp_bilateral 10,12,2,0,1,24,0")
        "-v - -gimp_bilateral 20,24,2,0,1,24,0")
    
    ConvolveLayer = pdb.gimp_layer_new_from_visible(inImage, inImage, "Visible Convolution Layer ")
    inImage.add_layer(ConvolveLayer, 0)
    ConvolveLayer.mode = NORMAL_MODE
    ConvolveLayer.opacity = 100.0
    
    # ##########################
    # Begin Jazzon's alterations
    # ##########################
    if width > height:
        aspectMult = float(width) / float(height)
        tWidth = width / inDivisor
        tHeight = int(tWidth / aspectMult)
    else:
        aspectMult = float(height) / float(width)
        tHeight = height / inDivisor
        tWidth = int(tHeight / aspectMult)
               
    if inDivisor != 1:            
        #pdb.gimp_image_scale(inImage, int(width/inDivisor), int(height/inDivisor))
        pdb.gimp_image_scale(inImage, tWidth, tHeight)
    # ########################
    # End Jazzon's alterations
    # ########################
        
    WhiteLayer1 = pdb.gimp_layer_new(inImage, width, height, type,
                                    "White Layer One", 50.00, NORMAL_MODE)
    inImage.add_layer(WhiteLayer1, 0)
    pdb.gimp_drawable_fill(WhiteLayer1, WHITE_FILL)
    For3dLayer = pdb.gimp_layer_new_from_visible(inImage, inImage, "For 3D Layer ")
    inImage.add_layer(For3dLayer, 0)
    For3dLayer.mode = NORMAL_MODE
    For3dLayer.opacity = 100.0
    # check image size
    if width > height :
        dwidth = min (width*2, 2048)
        # Altered by Jazzon
        dheight = int(dwidth / aspectMult)  # height*dwidth/width
    else:
        dheight = min (height*2, 2048)
        # Altered by Jazzon
        dwidth = int(dheight / aspectMult) # width*dheight/height
    #pdb.gimp_message("dwidth = "+str(dwidth)+", dheight = "+str(dheight))    
    # [G'MIC] 3d elevation: -gimp_elevation3d 100,4,1024,1024,1,0,0,0,45,0,0,-100,0.5,0.7,1,1
    pdb.plug_in_gmic(inImage, BilatLayer, 1,    # 1=input only active layer
        "-v - -gimp_elevation3d 100,4,"
        +str(dwidth)+","+str(dheight)
        +",1,0,0,0,45,0,0,-100,0.5,0.7,1,1")
    
    # added by Jazzon    
    pdb.gimp_layer_scale(
        pdb.gimp_image_get_layer_by_name(inImage, "For 3D Layer "),
        inImage.width, inImage.height, 0)
    
    BlackLayer = pdb.gimp_layer_new(inImage, width*2, height*2, type,
                                    "Black Layer", 100.00, NORMAL_MODE)
    inImage.add_layer(BlackLayer, 1)
    pdb.gimp_context_set_background((0,0,0))
    pdb.gimp_drawable_fill(BlackLayer, BACKGROUND_FILL)
    
    WireframeLayer = pdb.gimp_layer_new_from_visible(inImage, inImage, "Visible Wireframe Layer ")
    inImage.add_layer(WireframeLayer, 0)
    WireframeLayer.mode = NORMAL_MODE
    WireframeLayer.opacity = 100.0
    pdb.plug_in_autostretch_hsv(inImage, WireframeLayer)
    pdb.gimp_brightness_contrast(WireframeLayer, 0, inContrast)
    pdb.gimp_levels_stretch(WireframeLayer)
    pdb.gimp_desaturate_full(WireframeLayer, DESATURATE_LUMINOSITY)
    EqualizedLayer = pdb.gimp_layer_new_from_visible(inImage, inImage, "Equalized Wireframe Layer ")
    inImage.add_layer(EqualizedLayer, 0)
    EqualizedLayer.mode = DODGE_MODE
    EqualizedLayer.opacity = 35.0
    pdb.gimp_equalize(EqualizedLayer, False)
    
    pdb.gimp_image_scale(inImage, width, height)
    
    if (doColour == True) :
        SavedLayer = pdb.gimp_layer_new(inImage, width, height, type,
                                    "Saved Layer", 100.00, COLOR_MODE)
        inImage.add_layer(SavedLayer, 0)
        pdb.gimp_floating_sel_anchor(
            pdb.gimp_edit_named_paste(SavedLayer, origBufferName, False))
        pdb.plug_in_gauss_iir2(inImage, SavedLayer, int(width/70), int(height/70))
        
    
    if (doFlatten == True) :
        pdb.gimp_image_flatten(inImage)
        
    pdb.gimp_displays_flush
   
    # Restore 
    pdb.gimp_image_undo_group_end(inImage)
    pdb.gimp_context_pop
    pdb.gimp_context_set_background(currBG)
    pdb.gimp_context_set_foreground(currFG)
    return
    

# This is the plugin registration function
register(
    "WireframeMap_Lyle (bug fix)",    
    "to derive a 2d wireframe texture from an image",   
    "This script takes an input image and transform it into a 2d wireframe texture",
    "Diego", 
    "Diego Nassetti on a Lyle's tutorial", 
    "January 2016",
    "iaz_WireframeMap_Lyle", 
    "", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "divisor", "Divisor for downsizing", 3, (1, 8, 1)),
      (PF_ADJUSTMENT, "contrast", "Contrast adjustment", 30, (15, 75, 5)),
      (PF_TOGGLE, "colours", "Restore colours Image?", False),
      (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ], 
    [],
    WireframeMap_Lyle,
    menu="<Image>/Diego/Wiremap",
    )

main()
